@@ -51,6 +51,10 @@
  * but provides the basic signaling and operation control so the miners
  * implementing this class are properly recognized by Tracker, and can be
  * controlled properly by external means such as #TrackerMinerManager.
+ *
+ * #TrackerMiner implements the #GInitable interface, and thus, all objects of
+ * types inheriting from #TrackerMiner must be initialized with g_initable_init()
+ * just after creation (or directly created with g_initable_new()).
  **/
 
 #define TRACKER_MINER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER, TrackerMinerPrivate))
@@ -105,7 +109,6 @@ struct _TrackerMinerPrivate {
 	GDBusNodeInfo *introspection_data;
 	guint watch_name_id;
 	guint registration_id;
-	guint own_id;
 	gchar *full_name;
 	gchar *full_path;
 };
@@ -135,21 +138,56 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-static void       miner_set_property           (GObject       *object,
-                                                guint          param_id,
-                                                const GValue  *value,
-                                                GParamSpec    *pspec);
-static void       miner_get_property           (GObject       *object,
-                                                guint          param_id,
-                                                GValue        *value,
-                                                GParamSpec    *pspec);
-static void       miner_finalize               (GObject       *object);
-static void       miner_constructed            (GObject       *object);
-static void       pause_data_destroy           (gpointer       data);
-static PauseData *pause_data_new               (const gchar   *application,
-                                                const gchar   *reason);
+static void       miner_set_property           (GObject                *object,
+                                                guint                   param_id,
+                                                const GValue           *value,
+                                                GParamSpec             *pspec);
+static void       miner_get_property           (GObject                *object,
+                                                guint                   param_id,
+                                                GValue                 *value,
+                                                GParamSpec             *pspec);
+static void       miner_finalize               (GObject                *object);
+static void       miner_initable_iface_init    (GInitableIface         *iface);
+static gboolean   miner_initable_init          (GInitable              *initable,
+                                                GCancellable           *cancellable,
+                                                GError                **error);
+static void       pause_data_destroy           (gpointer                data);
+static PauseData *pause_data_new               (const gchar            *application,
+                                                const gchar            *reason);
+static void       handle_method_call           (GDBusConnection        *connection,
+                                                const gchar            *sender,
+                                                const gchar            *object_path,
+                                                const gchar            *interface_name,
+                                                const gchar            *method_name,
+                                                GVariant               *parameters,
+                                                GDBusMethodInvocation  *invocation,
+                                                gpointer                user_data);
+static GVariant  *handle_get_property          (GDBusConnection        *connection,
+                                                const gchar            *sender,
+                                                const gchar            *object_path,
+                                                const gchar            *interface_name,
+                                                const gchar            *property_name,
+                                                GError                **error,
+                                                gpointer                user_data);
+static gboolean   handle_set_property          (GDBusConnection        *connection,
+                                                const gchar            *sender,
+                                                const gchar            *object_path,
+                                                const gchar            *interface_name,
+                                                const gchar            *property_name,
+                                                GVariant               *value,
+                                                GError                **error,
+                                                gpointer                user_data);
+static void       on_tracker_store_appeared    (GDBusConnection        *connection,
+                                                const gchar            *name,
+                                                const gchar            *name_owner,
+                                                gpointer                user_data);
+static void       on_tracker_store_disappeared (GDBusConnection        *connection,
+                                                const gchar            *name,
+                                                gpointer                user_data);
 
-G_DEFINE_ABSTRACT_TYPE (TrackerMiner, tracker_miner, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (TrackerMiner, tracker_miner, G_TYPE_OBJECT,
+                                  G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+                                                         miner_initable_iface_init));
 
 static void
 tracker_miner_class_init (TrackerMinerClass *klass)
@@ -159,7 +197,6 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	object_class->set_property = miner_set_property;
 	object_class->get_property = miner_get_property;
 	object_class->finalize     = miner_finalize;
-	object_class->constructed  = miner_constructed;
 
 	/**
 	 * TrackerMiner::started:
@@ -168,6 +205,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * the ::started signal is emitted in the miner
 	 * right after it has been started through
 	 * tracker_miner_start().
+	 *
+	 * Since: 0.8
 	 **/
 	signals[STARTED] =
 		g_signal_new ("started",
@@ -184,6 +223,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * the ::stopped signal is emitted in the miner
 	 * right after it has been stopped through
 	 * tracker_miner_stop().
+	 *
+	 * Since: 0.8
 	 **/
 	signals[STOPPED] =
 		g_signal_new ("stopped",
@@ -201,6 +242,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * there is any reason to pause, either
 	 * internal (through tracker_miner_pause()) or
 	 * external (through DBus, see #TrackerMinerManager).
+	 *
+	 * Since: 0.8
 	 **/
 	signals[PAUSED] =
 		g_signal_new ("paused",
@@ -217,6 +260,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * the ::resumed signal is emitted whenever
 	 * all reasons to pause have disappeared, see
 	 * tracker_miner_resume() and #TrackerMinerManager.
+	 *
+	 * Since: 0.8
 	 **/
 	signals[RESUMED] =
 		g_signal_new ("resumed",
@@ -236,6 +281,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * to indicate progress about the data mining process. @status will
 	 * contain a translated string with the current miner status and @progress
 	 * will indicate how much has been processed so far.
+	 *
+	 * Since: 0.8
 	 **/
 	signals[PROGRESS] =
 		g_signal_new ("progress",
@@ -256,6 +303,8 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	 * the ::ignore-next-update signal is emitted in the miner
 	 * right after it has been asked to mark @urls as to ignore on next update
 	 * through tracker_miner_ignore_next_update().
+	 *
+	 * Since: 0.8
 	 **/
 	signals[IGNORE_NEXT_UPDATE] =
 		g_signal_new ("ignore-next-update",
@@ -296,22 +345,133 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	miner_error_quark = g_quark_from_static_string ("TrackerMiner");
 }
 
+static void
+miner_initable_iface_init (GInitableIface *iface)
+{
+	iface->init = miner_initable_init;
+}
+
+static gboolean
+miner_initable_init (GInitable     *initable,
+                     GCancellable  *cancellable,
+                     GError       **error)
+{
+	TrackerMiner *miner = TRACKER_MINER (initable);
+	GError *inner_error = NULL;
+	GVariant *reply;
+	guint32 rval;
+	GDBusInterfaceVTable interface_vtable = {
+		handle_method_call,
+		handle_get_property,
+		handle_set_property
+	};
+
+	/* Try to get SPARQL connection... */
+	miner->private->connection = tracker_sparql_connection_get (NULL, &inner_error);
+	if (!miner->private->connection) {
+		g_propagate_error (error, inner_error);
+		return FALSE;
+	}
+
+	/* Try to get DBus connection... */
+	miner->private->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &inner_error);
+	if (!miner->private->d_connection) {
+		g_propagate_error (error, inner_error);
+		return FALSE;
+	}
+
+	/* Setup introspection data */
+	miner->private->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
+
+	/* Check miner has a proper name */
+	if (!miner->private->name) {
+		g_set_error (error,
+		             TRACKER_MINER_ERROR,
+		             0,
+		             "Miner '%s' should have been given a name, bailing out",
+		             G_OBJECT_TYPE_NAME (miner));
+		return FALSE;
+	}
+
+	/* Setup full name */
+	miner->private->full_name = g_strconcat (TRACKER_MINER_DBUS_NAME_PREFIX,
+	                                         miner->private->name,
+	                                         NULL);
+
+	/* Register the D-Bus object */
+	miner->private->full_path = g_strconcat (TRACKER_MINER_DBUS_PATH_PREFIX,
+	                                         miner->private->name,
+	                                         NULL);
+
+	g_message ("Registering D-Bus object...");
+	g_message ("  Path:'%s'", miner->private->full_path);
+	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (miner));
+
+	miner->private->registration_id =
+		g_dbus_connection_register_object (miner->private->d_connection,
+		                                   miner->private->full_path,
+	                                       miner->private->introspection_data->interfaces[0],
+	                                       &interface_vtable,
+	                                       miner,
+	                                       NULL,
+		                                   &inner_error);
+	if (inner_error) {
+		g_propagate_error (error, inner_error);
+		g_prefix_error (error,
+		                "Could not register the D-Bus object '%s'. ",
+		                miner->private->full_path);
+		return FALSE;
+	}
+
+	/* Request the D-Bus name */
+	reply = g_dbus_connection_call_sync (miner->private->d_connection,
+	                                     "org.freedesktop.DBus",
+	                                     "/org/freedesktop/DBus",
+	                                     "org.freedesktop.DBus",
+	                                     "RequestName",
+	                                     g_variant_new ("(su)",
+	                                                    miner->private->full_name,
+	                                                    0x4 /* DBUS_NAME_FLAG_DO_NOT_QUEUE */),
+	                                     G_VARIANT_TYPE ("(u)"),
+	                                     0, -1, NULL, &inner_error);
+	if (inner_error) {
+		g_propagate_error (error, inner_error);
+		g_prefix_error (error,
+		                "Could not acquire name:'%s'. ",
+		                miner->private->full_name);
+		return FALSE;
+	}
+
+	g_variant_get (reply, "(u)", &rval);
+	g_variant_unref (reply);
+
+	if (rval != 1 /* DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER */) {
+		g_set_error (error,
+		             TRACKER_MINER_ERROR,
+		             0,
+		             "D-Bus service name:'%s' is already taken, "
+		             "perhaps the application is already running?",
+		             miner->private->full_name);
+		return FALSE;
+	}
+
+	miner->private->watch_name_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+	                                                  TRACKER_SERVICE,
+	                                                  G_BUS_NAME_WATCHER_FLAGS_NONE,
+	                                                  on_tracker_store_appeared,
+	                                                  on_tracker_store_disappeared,
+	                                                  miner,
+	                                                  NULL);
+
+	return TRUE;
+}
+
 static void
 tracker_miner_init (TrackerMiner *miner)
 {
 	TrackerMinerPrivate *priv;
-	GError *error = NULL;
-
 	miner->private = priv = TRACKER_MINER_GET_PRIVATE (miner);
 
-	priv->connection = tracker_sparql_connection_get (NULL, &error);
-
-	/* If we couldn't get a proper SPARQL connection, log error and abort */
-	if (!priv->connection) {
-		g_error ("Couldn't get a proper SPARQL connection (%s), exiting...",
-		         error ? error->message : "Unknown error");
-	}
-
 	priv->pauses = g_hash_table_new_full (g_direct_hash,
 	                                      g_direct_equal,
 	                                      NULL,
@@ -376,7 +536,7 @@ miner_set_property (GObject      *object,
 		 * we set it last.
 		 *
 		 * Only notify 1% changes
-		 */ 
+		 */
 		if (new_progress == miner->private->progress) {
 			/* Same, do nothing */
 			break;
@@ -451,6 +611,8 @@ pause_data_destroy (gpointer data)
  * Returns the #GQuark used to identify miner errors in GError structures.
  *
  * Returns: the error #GQuark
+ *
+ * Since: 0.8
  **/
 GQuark
 tracker_miner_error_quark (void)
@@ -463,6 +625,8 @@ tracker_miner_error_quark (void)
  * @miner: a #TrackerMiner
  *
  * Tells the miner to start processing data.
+ *
+ * Since: 0.8
  **/
 void
 tracker_miner_start (TrackerMiner *miner)
@@ -490,6 +654,8 @@ tracker_miner_start (TrackerMiner *miner)
  * @miner: a #TrackerMiner
  *
  * Tells the miner to stop processing data.
+ *
+ * Since: 0.8
  **/
 void
 tracker_miner_stop (TrackerMiner *miner)
@@ -518,6 +684,8 @@ tracker_miner_stop (TrackerMiner *miner)
  * @urls: the urls to mark as to ignore on next update
  *
  * Tells the miner to mark @urls are to ignore on next update.
+ *
+ * Since: 0.8
  **/
 void
 tracker_miner_ignore_next_update (TrackerMiner *miner,
@@ -535,6 +703,8 @@ tracker_miner_ignore_next_update (TrackerMiner *miner,
  * Returns #TRUE if the miner has been started.
  *
  * Returns: #TRUE if the miner is already started.
+ *
+ * Since: 0.8
  **/
 gboolean
 tracker_miner_is_started (TrackerMiner *miner)
@@ -544,6 +714,24 @@ tracker_miner_is_started (TrackerMiner *miner)
 	return miner->private->started;
 }
 
+/**
+ * tracker_miner_is_paused:
+ * @miner: a #TrackerMiner
+ *
+ * Returns #TRUE if the miner is paused.
+ *
+ * Returns: #TRUE if the miner is paused.
+ *
+ * Since: 0.10
+ **/
+gboolean
+tracker_miner_is_paused (TrackerMiner *miner)
+{
+	g_return_val_if_fail (TRACKER_IS_MINER (miner), TRUE);
+
+	return g_hash_table_size (miner->private->pauses) > 0 ? TRUE : FALSE;
+}
+
 static gint
 tracker_miner_pause_internal (TrackerMiner  *miner,
                               const gchar   *application,
@@ -604,6 +792,8 @@ tracker_miner_pause_internal (TrackerMiner  *miner,
  * operations. On failure @error will be set and -1 will be returned.
  *
  * Returns: The pause cookie ID.
+ *
+ * Since: 0.8
  **/
 gint
 tracker_miner_pause (TrackerMiner  *miner,
@@ -635,6 +825,8 @@ tracker_miner_pause (TrackerMiner  *miner,
  * operations until all pause requests have been resumed.
  *
  * Returns: #TRUE if the cookie was valid.
+ *
+ * Since: 0.8
  **/
 gboolean
 tracker_miner_resume (TrackerMiner  *miner,
@@ -668,12 +860,70 @@ tracker_miner_resume (TrackerMiner  *miner,
 	return TRUE;
 }
 
+/**
+ * tracker_miner_get_connection:
+ * @miner: a #TrackerMiner
+ *
+ * Gets the #TrackerSparqlConnection initialized by @miner
+ *
+ * Returns: a #TrackerSparqlConnection.
+ *
+ * Since: 0.10
+ **/
 TrackerSparqlConnection *
 tracker_miner_get_connection (TrackerMiner *miner)
 {
 	return miner->private->connection;
 }
 
+/**
+ * tracker_miner_get_dbus_connection:
+ * @miner: a #TrackerMiner
+ *
+ * Gets the #GDBusConnection initialized by @miner
+ *
+ * Returns: a #GDBusConnection.
+ *
+ * Since: 0.10
+ **/
+GDBusConnection *
+tracker_miner_get_dbus_connection (TrackerMiner *miner)
+{
+	return miner->private->d_connection;
+}
+
+/**
+ * tracker_miner_get_dbus_full_name:
+ * @miner: a #TrackerMiner
+ *
+ * Gets the DBus name registered by @miner
+ *
+ * Returns: a constant string which should not be modified by the caller.
+ *
+ * Since: 0.10
+ **/
+G_CONST_RETURN gchar *
+tracker_miner_get_dbus_full_name (TrackerMiner *miner)
+{
+	return miner->private->full_name;
+}
+
+/**
+ * tracker_miner_get_dbus_full_path:
+ * @miner: a #TrackerMiner
+ *
+ * Gets the DBus path registered by @miner
+ *
+ * Returns: a constant string which should not be modified by the caller.
+ *
+ * Since: 0.10
+ **/
+G_CONST_RETURN gchar *
+tracker_miner_get_dbus_full_path (TrackerMiner *miner)
+{
+	return miner->private->full_path;
+}
+
 static void
 miner_finalize (GObject *object)
 {
@@ -683,10 +933,6 @@ miner_finalize (GObject *object)
 		g_bus_unwatch_name (miner->private->watch_name_id);
 	}
 
-	if (miner->private->own_id != 0) {
-		g_bus_unown_name (miner->private->own_id);
-	}
-
 	if (miner->private->registration_id != 0) {
 		g_dbus_connection_unregister_object (miner->private->d_connection,
 		                                     miner->private->registration_id);
@@ -709,7 +955,9 @@ miner_finalize (GObject *object)
 		g_object_unref (miner->private->connection);
 	}
 
-	g_hash_table_unref (miner->private->pauses);
+	if (miner->private->pauses) {
+		g_hash_table_unref (miner->private->pauses);
+	}
 
 	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
 }
@@ -985,80 +1233,3 @@ on_tracker_store_disappeared (GDBusConnection *connection,
 		}
 	}
 }
-
-static void
-miner_constructed (GObject *object)
-{
-	TrackerMiner *miner;
-	gchar *name, *full_path, *full_name;
-	GError *error = NULL;
-	GDBusInterfaceVTable interface_vtable = {
-		handle_method_call,
-		handle_get_property,
-		handle_set_property
-	};
-
-	miner = TRACKER_MINER (object);
-	miner->private->d_connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
-
-	if (!miner->private->d_connection) {
-		g_critical ("Could not connect to the D-Bus session bus, %s",
-		            error ? error->message : "no error given.");
-		g_clear_error (&error);
-		return;
-	}
-
-	miner->private->introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
-
-	g_object_get (miner, "name", &name, NULL);
-
-	if (!name) {
-		g_critical ("Miner '%s' should have been given a name, bailing out",
-		            G_OBJECT_TYPE_NAME (miner));
-		g_assert_not_reached ();
-	}
-
-	full_name = g_strconcat (TRACKER_MINER_DBUS_NAME_PREFIX, name, NULL);
-
-	miner->private->own_id = g_bus_own_name_on_connection (miner->private->d_connection,
-	                                                       full_name,
-	                                                       G_BUS_NAME_OWNER_FLAGS_NONE,
-	                                                       NULL, NULL, NULL, NULL);
-	miner->private->full_name = full_name;
-
-	/* Register the service name for the miner */
-	full_path = g_strconcat (TRACKER_MINER_DBUS_PATH_PREFIX, name, NULL);
-
-	g_message ("Registering D-Bus object...");
-	g_message ("  Path:'%s'", full_path);
-	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (miner));
-
-	miner->private->registration_id =
-		g_dbus_connection_register_object (miner->private->d_connection,
-	                                       full_path,
-	                                       miner->private->introspection_data->interfaces[0],
-	                                       &interface_vtable,
-	                                       miner,
-	                                       NULL,
-	                                       &error);
-
-	if (error) {
-		g_critical ("Could not register the D-Bus object %s, %s",
-		            full_path,
-		            error ? error->message : "no error given.");
-		g_clear_error (&error);
-		return;
-	}
-
-	g_free (name);
-
-	miner->private->full_path = full_path;
-
-	miner->private->watch_name_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
-	                                                  TRACKER_SERVICE,
-	                                                  G_BUS_NAME_WATCHER_FLAGS_NONE,
-	                                                  on_tracker_store_appeared,
-	                                                  on_tracker_store_disappeared,
-	                                                  miner,
-	                                                  NULL);
-}
